@@ -238,8 +238,8 @@ Bhattacharya <- function(param, n_rnorm = 1000, savePlots = FALSE){
       sum.m.co1 <- summary(m.co1)
       a.co1 <- sum.m.co1$coefficients[1]
       b.co1 <- sum.m.co1$coefficients[2]
-      l.mean.co1 <- -a.co1/b.co1   #mean length: L(mean)(N1) = -a/b
-      s.co1 <- sqrt(-1/b.co1)      #standard deviation: s(N1) = sqrt(-1/b)
+        l.mean.co1 <- -a.co1/b.co1        ## mean length: L(mean)(N1) = -a/b p.83
+        s.co1 <- sqrt(-interval/b.co1)    ## standard deviation: s(N1) = sqrt(-dL/b) p.83
 
       a.b.list[[xy]] <- c(a.co1,b.co1)
       l.s.list[[xy]] <- c(l.mean.co1,s.co1)
@@ -424,7 +424,7 @@ Bhattacharya <- function(param, n_rnorm = 1000, savePlots = FALSE){
     # seperation index (SI)
     SIs <- rep(NA,length(l.s.df[,1]))
     for(i in 1:(length(l.s.df[,1])-1)){
-      SIs[i] <- abs(l.s.df[i+1,2]-l.s.df[i,2]) / abs(l.s.df[i+1,3]-l.s.df[i,3])
+      SIs[i] <- abs(l.s.df[i+1,2]-l.s.df[i,2]) / abs((l.s.df[i+1,3]+l.s.df[i,3])/2)
     }
     l.s.df <- cbind(l.s.df,SIs)
     colnames(l.s.df) <- c("Cohort","Lmean","SD","SI")
